if select(6, GetAddOnInfo("PitBull_" .. (debugstack():match("[d%.][d%.][O%.]ns\\PitBull\\(.-)\\") or ""))) ~= "MISSING" then return end

local VERSION = tonumber(("$Revision: 36067 $"):match("%d+"))

local PitBull = PitBull
local PitBull_RaidTarget = PitBull:NewModule("RaidTarget", "AceEvent-2.0")
local self = PitBull_RaidTarget
if PitBull.revision < VERSION then
	PitBull.version = "r" .. VERSION
	PitBull.revision = VERSION
	PitBull.date = ("$Date: 2007-05-21 02:37:16 -0400 (Mon, 21 May 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end
self.desc = "Show an icon on the unit frame based on which Raid Target it is."

local L = PitBull:L("PitBull-RaidTarget")

local new, del, newHash = PitBull.new, PitBull.del, PitBull.newHash
local newFrame, delFrame = PitBull.newFrame, PitBull.delFrame

PitBull_RaidTarget:RegisterPitBullChildFrames('raidTargetIcon')

function PitBull_RaidTarget:OnInitialize()
	self.db = PitBull:AcquireDBNamespace("RaidTarget")
	PitBull:RegisterDefaults("RaidTarget", "profile", {
		['*'] = {
			disable = false,
		}
	})
end

function PitBull_RaidTarget:OnEnable()
	self:RegisterEvent("RAID_TARGET_UPDATE")
	self:RegisterEvent("RAID_ROSTER_UPDATE", "RAID_TARGET_UPDATE")
	self:RegisterEvent("PARTY_MEMBERS_CHANGED", "RAID_TARGET_UPDATE")
end

function PitBull_RaidTarget:Update(unit, frame)
	local index = GetRaidTargetIndex(unit)
	if not frame.group then
		AceLibrary("AceConsole-2.0"):Print("Error in PitBull_RaidTarget, please report this to ckknight:", debugstack())
		return
	end
	if self.db.profile[frame.group].disable then
		index = nil
	end
	if index then
		if not frame.raidTargetIcon then
			frame.raidTargetIcon = newFrame("Texture", frame.overlay, "ARTWORK")
			frame.raidTargetIcon:SetTexture("Interface\\TargetingFrame\\UI-RaidTargetingIcons")
			PitBull:UpdateLayout(frame)
		end
		
		SetRaidTargetIconTexture(frame.raidTargetIcon, index)
	else
		if frame.raidTargetIcon then
			frame.raidTargetIcon = delFrame(frame.raidTargetIcon)
			
			PitBull:UpdateLayout(frame)
		end
	end
end

function PitBull_RaidTarget:RAID_TARGET_UPDATE()
	for unit, frame in PitBull:IterateNonWackyUnitFrames() do
		self:Update(unit, frame)
	end
end

function PitBull_RaidTarget:OnUpdateFrame(unit, frame)
	self:Update(unit, frame)
end

function PitBull_RaidTarget:OnClearUnitFrame(unit, frame)
	if frame.raidTargetIcon then
		frame.raidTargetIcon = delFrame(frame.raidTargetIcon)
	end
end

function PitBull_RaidTarget:OnUnknownLayout(unit, frame)
	frame.raidTargetIcon:SetPoint("CENTER", frame, "TOP", 0, -3)
	frame.raidTargetIcon:SetWidth(18)
	frame.raidTargetIcon:SetHeight(18)
end

local function getEnable(group)
	return not PitBull_RaidTarget.db.profile[group].disable
end
local function setEnable(group, value)
	PitBull_RaidTarget.db.profile[group].disable = not value
	
	for unit, frame in PitBull:IterateUnitFramesByGroup(group) do
		PitBull_RaidTarget:Update(unit, frame)
	end
end
PitBull_RaidTarget:RegisterPitBullOptionsMethod(function(group)
	return {
		type = "toggle",
		name = L["Enable raid target icon"],
		desc = L["Enable raid target icon display on the unit frames for this unit type."],
		get = getEnable,
		set = setEnable,
		passValue = group,
	}
end)
